﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=9.85
@EndOfDesignText@
#Region Shared Files
#CustomBuildAction: folders ready, %WINDIR%\System32\Robocopy.exe,"..\..\Shared Files" "..\Files"
'Ctrl + click to sync files: ide://run?file=%WINDIR%\System32\Robocopy.exe&args=..\..\Shared+Files&args=..\Files&FilesSync=True
#End Region

'Ctrl + click to export as zip: ide://run?File=%B4X%\Zipper.jar&Args=Project.zip
'See the list of page related events in the B4XPagesManager object. The event name is B4XPage.

Sub Class_Globals
	
	'Constants
	Private Const APP_TITLE As String = "DMX Client"
	Private Const APP_VERSION As String = "200240607"
	Private Const APP_INFO As String = "Example controlling an LED PAR in DMX512 Mode using the DMX Controller Arduino MEGA with B4R."
	Private Const APP_HINT As String = "Ensure to set the dimmer first after connecting to the DMX Controller."

	'UI	
	Private Root As B4XView
	Private xui As XUI
	Private btnConnect As B4XView
	Private lblConnectStatus As B4XView
	Private btnSetChannelValue As B4XView
	Private spAddress As Spinner
	Private SliderLevelBlue As SliderLevel
	Private SliderLevelDimmer As SliderLevel
	Private SliderLevelGreen As SliderLevel
	Private SliderLevelRed As SliderLevel
	Private spChannel As Spinner
	Private tfHostIP As B4XView
	Private tfValue As B4XView
	Private lvLog As ListView
	Private blnClearLog As B4XView
	Private lblInfo As B4XView
	
	'Communication
	Private sock As Socket
	Private isConnected As Boolean
	Private astream As AsyncStreams
   
	Private Const HOST_IP As String = "dmx-controller-ip"
	Private Const HOST_PORT As Int = 80

	'LED address, channel range 0-512, level range 0-100, value range 0-255 (sent to the DMX interface)
	Private LED_Address As Int = 1
	Private Const CHANNEL_MIN As Int = 1
	Private Const CHANNEL_MAX As Int = 512
	Private Const LEVEL_MIN As Int = 0
	Private Const LEVEL_MAX As Int = 100
	Private Const VALUE_MIN As Int = 0
	Private Const VALUE_MAX As Int = 255
End Sub

Public Sub Initialize
	B4XPages.GetManager.LogEvents = True
End Sub

'This event will be called once, before the page becomes visible.
Private Sub B4XPage_Created (Root1 As B4XView)
	Root = Root1
	Root.LoadLayout("MainPage")
	
	'Set mainpage properties
	B4XPages.SetTitle(Me, $"${APP_TITLE} - ${APP_VERSION}"$)
	B4XPages.GetNativeParent(Me).Resizable = False    
	
	'Set defaults
	tfHostIP.Text = HOST_IP
	lblConnectStatus.Text = "Disconnected"
	lblInfo.Text = $"${APP_INFO}${CRLF}${APP_HINT}"$
	
	spAddress.Value = LED_Address

	CallSubDelayed(Me, "Set_SliderLevel_Properties")

	'Connect the socket 
	'sock_Connect
End Sub

'NOT USED = just to show how to use GetView 0=LabelText,1=SliderLevel,2=LabelLevel
Private Sub Set_SliderLevel_Properties
'	Dim LabelText As B4XView = SliderLevelDimmer.mBase.GetView(0)
'	LabelText.TextColor = xui.Color_Red
End Sub

Private Sub btnConnect_Click
	If Not(isConnected) Then
		sock_Connect
	End If
End Sub

Private Sub spAddress_ValueChanged (Value As Object)
	LED_Address = Value
End Sub

Private Sub SliderLevelDimmer_ValueChange (Value As Double)
	Set_Channel_Value(LED_Address, Value)
End Sub

Private Sub SliderLevelRed_ValueChange (Value As Double)
	Set_Channel_Value(LED_Address + 1, Value)
End Sub

Private Sub SliderLevelGreen_ValueChange (Value As Double)
	Set_Channel_Value(LED_Address + 2, Value)
End Sub

Private Sub SliderLevelBlue_ValueChange (Value As Double)
	Set_Channel_Value(LED_Address + 3, Value)
End Sub

Private Sub spChannel_ValueChanged (Value As Object)
	Log($"spChannel_value_Changed: ${Value}"$)
End Sub

Private Sub tfValue_Action
	btnSetChannelValue_Click
End Sub

Private Sub btnSetChannelValue_Click
	Set_Channel_Value(spChannel.Value, tfValue.Text.As(Int))
End Sub

Private Sub blnClearLog_Click
	lvLog.Items.Clear	
End Sub

#Region Communication
Sub sock_Connect
	ToLog("[sock_Connect] Connecting")
	sock.Initialize("sock")
	sock.Connect(tfHostIP.Text, HOST_PORT, 10000)
End Sub

'Handle socket connection
Sub sock_Connected (Successful As Boolean)
	ToLog("[sock_Connected] " & Successful)
	isConnected = Successful
	If Successful Then
		astream.Initialize(sock.InputStream, sock.OutputStream, "astream")
		lblConnectStatus.Text = "Connected"
	Else
		ToLog("[ERROR] Can not connect to the DMX Controller")
		lblConnectStatus.Text = "Disconnected"
	End If
End Sub

'Log data received from the Arduino DMX RESTFul server.
'The data comes in as chunks.
'Logging is NOT used.
Sub astream_NewData (Buffer() As Byte)
	'Dim bc As ByteConverter	'Lib ByteConverter
	'Log($"[astream_NewData]${bc.StringFromBytes(Buffer,"UTF-8")}"$)
End Sub

'Set DMX channel and value by sending async write to the DMX RESTFul server.
Sub Set_Channel_Value(channel As Int, value As Int)
	'Do nothing if not connected to the DMX RESTFul server
	If Not(isConnected) Then Return

	'Map from level 0-100 to value 0-255
	value = MapRange(value, LEVEL_MIN, LEVEL_MAX, VALUE_MIN, VALUE_MAX)
	
	'Check if the value is in range and write the command. 
	'The command is an HTTP GET command with arguments.
	If (channel >= CHANNEL_MIN And channel <= CHANNEL_MAX) And (value >= VALUE_MIN And value <= VALUE_MAX) Then
		astream.Write($"/channel=${channel}&value=${value}&disconnect=0"$.GetBytes("UTF8"))
		ToLog($"[Set_Channel_Value] Set channel=${channel},value=${value}"$)
	Else
		ToLog($"[Set_Channel_Value][ERROR] Can not set channel ${channel} or value ${value} for the DMX Controller - Check range"$)
	End If
End Sub
#End Region

#Region Helpers
Private Sub MapRange(Value As Int, FromLow As Int, FromHigh As Int, ToLow As Int, ToHigh As Int) As Int
	Return ToLow + (ToHigh - ToLow) * ((Value - FromLow) / (FromHigh - FromLow))
End Sub

Private Sub ToLog(msg As String)
	msg = $"${DateTime.Time(DateTime.Now)} ${msg}"$
	lvLog.Items.InsertAt(0, msg)
	Log(msg)
End Sub
#End Region
